@@ -7,7 +7,10 @@ After obtaining an auth code, the web server will automatically shut down.
 """
 import logging
 import socket
+import sys
 from string import Template
+import threading
+import time
 
 try:  # Python 3
     from http.server import HTTPServer, BaseHTTPRequestHandler
@@ -33,9 +36,39 @@ def obtain_auth_code(listen_port, auth_uri=None):  # Historically only used in t
             ).get("code")
 
 
-def _browse(auth_uri):  # throws ImportError, possibly webbrowser.Error in future
+def is_wsl():
+    # "Official" way of detecting WSL: https://github.com/Microsoft/WSL/issues/423#issuecomment-221627364
+    # Run `uname -a` to get 'release' without python
+    #   - WSL 1: '4.4.0-19041-Microsoft'
+    #   - WSL 2: '4.19.128-microsoft-standard'
+    import platform
+    uname = platform.uname()
+    platform_name = getattr(uname, 'system', uname[0]).lower()
+    release = getattr(uname, 'release', uname[2]).lower()
+    return platform_name == 'linux' and 'microsoft' in release
+
+
+def _browse(auth_uri, browser_name=None):  # throws ImportError, webbrowser.Error
+    """Browse uri with named browser. Default browser is customizable by $BROWSER"""
     import webbrowser  # Lazy import. Some distro may not have this.
-    return webbrowser.open(auth_uri)  # Use default browser. Customizable by $BROWSER
+    if browser_name:
+        browser_opened = webbrowser.get(browser_name).open(auth_uri)
+    else:
+        # This one can survive BROWSER=nonexist, while get(None).open(...) can not
+        browser_opened = webbrowser.open(auth_uri)
+
+    # In WSL which doesn't have www-browser, try launching browser with PowerShell
+    if not browser_opened and is_wsl():
+        try:
+            import subprocess
+            # https://docs.microsoft.com/en-us/powershell/module/microsoft.powershell.core/about/about_powershell_exe
+            # Ampersand (&) should be quoted
+            exit_code = subprocess.call(
+                ['powershell.exe', '-NoProfile', '-Command', 'Start-Process "{}"'.format(auth_uri)])
+            browser_opened = exit_code == 0
+        except FileNotFoundError:  # WSL might be too old
+            pass
+    return browser_opened
 
 
 def _qs2kv(qs):
@@ -71,7 +104,17 @@ class _AuthCodeHandler(BaseHTTPRequestHandler):
         logger.debug(format, *args)  # To override the default log-to-stderr behavior
 
 
-class _AuthCodeHttpServer(HTTPServer):
+class _AuthCodeHttpServer(HTTPServer, object):
+    def __init__(self, server_address, *args, **kwargs):
+        _, port = server_address
+        if port and (sys.platform == "win32" or is_wsl()):
+            # The default allow_reuse_address is True. It works fine on non-Windows.
+            # On Windows, it undesirably allows multiple servers listening on same port,
+            # yet the second server would not receive any incoming request.
+            # So, we need to turn it off.
+            self.allow_reuse_address = False
+        super(_AuthCodeHttpServer, self).__init__(server_address, *args, **kwargs)
+
     def handle_timeout(self):
         # It will be triggered when no request comes in self.timeout seconds.
         # See https://docs.python.org/3/library/socketserver.html#socketserver.BaseServer.handle_timeout
@@ -87,7 +130,7 @@ class _AuthCodeHttpServer6(_AuthCodeHttpServer):
 
 class AuthCodeReceiver(object):
     # This class has (rather than is) an _AuthCodeHttpServer, so it does not leak API
-    def __init__(self, port=None):
+    def __init__(self, port=None, scheduled_actions=None):
         """Create a Receiver waiting for incoming auth response.
 
         :param port:
@@ -96,6 +139,12 @@ class AuthCodeReceiver(object):
             If your Identity Provider supports dynamic port, you can use port=0 here.
             Port 0 means to use an arbitrary unused port, per this official example:
             https://docs.python.org/2.7/library/socketserver.html#asynchronous-mixins
+
+        :param scheduled_actions:
+            For example, if the input is
+            ``[(10, lambda: print("Got stuck during sign in? Call 800-000-0000"))]``
+            then the receiver would call that lambda function after
+            waiting the response for 10 seconds.
         """
         address = "127.0.0.1"  # Hardcode, for now, Not sure what to expose, yet.
             # Per RFC 8252 (https://tools.ietf.org/html/rfc8252#section-8.3):
@@ -109,20 +158,19 @@ class AuthCodeReceiver(object):
             #   When this server physically listens to a specific IP (as it should),
             #   you will still be able to specify your redirect_uri using either
             #   IP (e.g. 127.0.0.1) or localhost, whichever matches your registration.
+        self._scheduled_actions = sorted(scheduled_actions or [])  # Make a copy
         Server = _AuthCodeHttpServer6 if ":" in address else _AuthCodeHttpServer
             # TODO: But, it would treat "localhost" or "" as IPv4.
             # If pressed, we might just expose a family parameter to caller.
         self._server = Server((address, port or 0), _AuthCodeHandler)
+        self._closing = False
 
     def get_port(self):
         """The port this server actually listening to"""
         # https://docs.python.org/2.7/library/socketserver.html#SocketServer.BaseServer.server_address
         return self._server.server_address[1]
 
-    def get_auth_response(self, auth_uri=None, timeout=None, state=None,
-            welcome_template=None, success_template=None, error_template=None,
-            auth_uri_callback=None,
-            ):
+    def get_auth_response(self, timeout=None, **kwargs):
         """Wait and return the auth response. Raise RuntimeError when timeout.
 
         :param str auth_uri:
@@ -148,6 +196,12 @@ class AuthCodeReceiver(object):
             A function with the shape of lambda auth_uri: ...
             When a browser was unable to be launch, this function will be called,
             so that the app could tell user to manually visit the auth_uri.
+        :param str browser_name:
+            If you did
+            ``webbrowser.register("xyz", None, BackgroundBrowser("/path/to/browser"))``
+            beforehand, you can pass in the name "xyz" to use that browser.
+            The default value ``None`` means using default browser,
+            which is customizable by env var $BROWSER.
         :return:
             The auth response of the first leg of Auth Code flow,
             typically {"code": "...", "state": "..."} or {"error": "...", ...}
@@ -155,6 +209,41 @@ class AuthCodeReceiver(object):
             and https://openid.net/specs/openid-connect-core-1_0.html#AuthResponse
             Returns None when the state was mismatched, or when timeout occurred.
         """
+        # Historically, the _get_auth_response() uses HTTPServer.handle_request(),
+        # because its handle-and-retry logic is conceptually as easy as a while loop.
+        # Also, handle_request() honors server.timeout setting, and CTRL+C simply works.
+        # All those are true when running on Linux.
+        #
+        # However, the behaviors on Windows turns out to be different.
+        # A socket server waiting for request would freeze the current thread.
+        # Neither timeout nor CTRL+C would work. End user would have to do CTRL+BREAK.
+        # https://stackoverflow.com/questions/1364173/stopping-python-using-ctrlc
+        #
+        # The solution would need to somehow put the http server into its own thread.
+        # This could be done by the pattern of ``http.server.test()`` which internally
+        # use ``ThreadingHTTPServer.serve_forever()`` (only available in Python 3.7).
+        # Or create our own thread to wrap the HTTPServer.handle_request() inside.
+        result = {}  # A mutable object to be filled with thread's return value
+        t = threading.Thread(
+            target=self._get_auth_response, args=(result,), kwargs=kwargs)
+        t.daemon = True  # So that it won't prevent the main thread from exiting
+        t.start()
+        begin = time.time()
+        while (time.time() - begin < timeout) if timeout else True:
+            time.sleep(1)  # Short detection interval to make happy path responsive
+            if not t.is_alive():  # Then the thread has finished its job and exited
+                break
+            while (self._scheduled_actions
+                    and time.time() - begin > self._scheduled_actions[0][0]):
+                _, callback = self._scheduled_actions.pop(0)
+                callback()
+        return result or None
+
+    def _get_auth_response(self, result, auth_uri=None, timeout=None, state=None,
+            welcome_template=None, success_template=None, error_template=None,
+            auth_uri_callback=None,
+            browser_name=None,
+            ):
         welcome_uri = "http://localhost:{p}".format(p=self.get_port())
         abort_uri = "{loc}?error=abort".format(loc=welcome_uri)
         logger.debug("Abort by visit %s", abort_uri)
@@ -165,7 +254,7 @@ class AuthCodeReceiver(object):
             logger.info("Open a browser on this device to visit: %s" % _uri)
             browser_opened = False
             try:
-                browser_opened = _browse(_uri)
+                browser_opened = _browse(_uri, browser_name=browser_name)
             except:  # Had to use broad except, because the potential
                      # webbrowser.Error is purposely undefined outside of _browse().
                 # Absorb and proceed. Because browser could be manually run elsewhere.
@@ -192,7 +281,8 @@ class AuthCodeReceiver(object):
 
         self._server.timeout = timeout  # Otherwise its handle_timeout() won't work
         self._server.auth_response = {}  # Shared with _AuthCodeHandler
-        while True:
+        while not self._closing:  # Otherwise, the handle_request() attempt
+                                  # would yield noisy ValueError trace
             # Derived from
             # https://docs.python.org/2/library/basehttpserver.html#more-examples
             self._server.handle_request()
@@ -201,10 +291,11 @@ class AuthCodeReceiver(object):
                     logger.debug("State mismatch. Ignoring this noise.")
                 else:
                     break
-        return self._server.auth_response
+        result.update(self._server.auth_response)  # Return via writable result param
 
     def close(self):
         """Either call this eventually; or use the entire class as context manager"""
+        self._closing = True
         self._server.server_close()
 
     def __enter__(self):
@@ -245,4 +336,3 @@ if __name__ == '__main__':
             timeout=60,
             state=flow["state"],  # Optional
             ), indent=4))
-
